Object subclass: #ServerManager
	instanceVariableNames: 'mySocket anwers modelEvents interestedPartiesInModelEvents awaitingForAnswer '
	classVariableNames: 'Current '
	poolDictionaries: ''
	category: 'Scrabble-Client'!
!ServerManager commentStamp: '<historical>' prior: 0!
I am the in charge of communication with the Server!


!ServerManager methodsFor: 'client processes' stamp: 'JMV 7/20/1999 23:40'!
connectToServer: anIPAddress

	| myPort initialSocket addr |
	myPort _ 54324 + (100 atRandom).
	Transcript cr.
	Transcript show: 'Starting connection with Scrabble Server'; cr.
	Transcript show: 'Starting network'; cr.
	Socket initializeNetwork.
	addr _ anIPAddress.
	Transcript show: 'Connecting with Server'; cr.
	mySocket _ SimpleClientSocket new.
	mySocket listenOn: myPort.
	initialSocket _ SimpleClientSocket new.
	initialSocket connectTo: addr port: 54323.
	initialSocket waitForConnectionUntil: (Socket deadlineSecs: 16).
	(initialSocket isConnected) ifFalse: [
		self error: 'Initial Connection could not be entablished'].
	Transcript show: 'Initial Connection Entablished'; cr.

	initialSocket sendCommand: 
		NetNameResolver localAddressString, ' ', myPort asString.
	initialSocket waitForSendDoneUntil: initialSocket class standardDeadline.
	initialSocket closeAndDestroy.

	mySocket waitForConnectionUntil: (Socket deadlineSecs: 20).
	(mySocket isConnected) ifFalse: [
		self error: 'Connection could not be entablished '].
	Transcript show: 'Connection Entablished '; cr.

	self notAwaitingForAnswer.
	anwers _ SharedQueue new.
	modelEvents _ SharedQueue new.
	[ self receiveIncomingMessages ] forkAt: Processor userInterruptPriority.! !

!ServerManager methodsFor: 'client processes' stamp: 'JMV 7/9/1999 17:50'!
disconnectFromServer

	mySocket closeAndDestroy.
	mySocket _ nil.
	anwers _ nil.
	modelEvents _ nil.
	Transcript show: 'Connection with Server ended'; cr.! !

!ServerManager methodsFor: 'client processes' stamp: 'JMV 7/25/1999 14:33'!
dispatchPendingEvents
	"Dispatch model events"

	| messageString parts objectID event arguments interested |

	modelEvents notNil ifTrue: [
		[ modelEvents size > 0 ] whileTrue: [
			messageString _ modelEvents next.
			parts _ (messageString copyFrom: 5 to: messageString size-1)
				substrings: self separatorCharacter.
			objectID _ parts first.
			event _ (parts at: 2) asSymbol.
			arguments _ Array new: parts size - 2.
			1 to: arguments size do: [ :i |
				arguments at: i put: (parts at: i+2) ].
			interested _ self interestedPartiesInModelEvents at: objectID ifAbsent: [nil].
			interested notNil ifTrue: [
				interested eventOccuredIn: objectID name: event arguments: arguments ].
		].
	].! !

!ServerManager methodsFor: 'client processes' stamp: 'JMV 7/20/1999 23:39'!
processOneCycle
	"A chance to do internal processing"

	self isAwaitingForAnswer ifFalse: [
		self dispatchPendingEvents ].! !

!ServerManager methodsFor: 'client processes' stamp: 'JMV 7/21/1999 20:07'!
receiveIncomingMessages

	| messageString |
	[ mySocket isNil ] whileFalse: [
		mySocket dataAvailable ifTrue: [
			messageString _ mySocket getResponse.
			(messageString beginsWith: 'evt') ifTrue: [
				modelEvents nextPut: messageString ]
			ifFalse: [
				anwers nextPut: messageString ].
		].
		(Delay forMilliseconds: 20) wait.
	]! !


!ServerManager methodsFor: 'services' stamp: 'JMV 7/9/1999 17:35'!
sendMessageToVirtualPlayroom: aSymbol withArguments: aLiteralOrIDArray

	^ self sendTo: 'VirtualPlayroom' 
		message: aSymbol 
		withArguments: aLiteralOrIDArray! !

!ServerManager methodsFor: 'services' stamp: 'JMV 7/20/1999 23:36'!
sendTo: anObjectID message: aSymbol withArguments: aLiteralOrIDArray

	| messageString answer result |
	messageString _ anObjectID, self separatorCharacter asString, aSymbol asString.
	aLiteralOrIDArray do: [ :each |
		messageString _ messageString, self separatorCharacter asString, each asString ].
	self sendLine: messageString.
	self awaitingForAnswer.
	answer _ anwers next.
	self notAwaitingForAnswer.
	(answer beginsWith: 'ocl') ifTrue: [
		answer size > 5 ifTrue: [
			result _ ((answer copyFrom: 5 to: answer size-1) 
				substrings: self separatorCharacter) asOrderedCollection ]
		ifFalse: [
			result _ OrderedCollection new ] ]
	ifFalse: [
		answer size > 5 ifTrue: [
			result _ (answer copyFrom: 5 to: answer size-1) ]
		ifFalse: [
			result _ '' ] ].
	^ result! !


!ServerManager methodsFor: 'accessing' stamp: 'JMV 7/16/1999 18:00'!
interestedPartiesInModelEvents
	"Answer the dictionary of interested objects in any model event "

	interestedPartiesInModelEvents isNil ifTrue: [
		interestedPartiesInModelEvents _ "WeakValue"Dictionary new.
	].
	^ interestedPartiesInModelEvents! !

!ServerManager methodsFor: 'accessing' stamp: 'JMV 3/26/2000 09:15'!
separatorCharacter
	"Answer the character used as separator in TCP/IP messages"
	^ $\! !


!ServerManager methodsFor: 'private' stamp: 'JMV 7/20/1999 23:34'!
awaitingForAnswer

	awaitingForAnswer _ true! !

!ServerManager methodsFor: 'private' stamp: 'JMV 7/20/1999 23:40'!
isAwaitingForAnswer

	^ awaitingForAnswer! !

!ServerManager methodsFor: 'private' stamp: 'JMV 7/20/1999 23:34'!
notAwaitingForAnswer

	awaitingForAnswer _ false! !

!ServerManager methodsFor: 'private' stamp: 'JMV 7/9/1999 12:13'!
sendLine: aString

	mySocket sendCommand: aString! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ServerManager class
	instanceVariableNames: ''!

!ServerManager class methodsFor: 'instance creation' stamp: 'JMV 7/9/1999 11:42'!
current
	"Answer the active instance"

	Current isNil ifTrue: [
		Current _ self new ].
	^ Current! !
